@@ -1,4 +1,4 @@
-# Copyright 2019-2021 AstroLab Software
+# Copyright 2019-2022 AstroLab Software
 # Author: Julien Peloton
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -20,16 +20,53 @@ import numpy as np
 
 import os
 
-from fink_science.conversion import mag2fluxcal_snana
-
 from fink_science import __file__
-from fink_science.utilities import load_scikit_model
+
+from fink_utils.data.utils import format_data_as_snana
+from fink_utils.data.utils import load_scikit_model
+from fink_utils.xmatch.simbad import return_list_of_eg_host
+
 from fink_science.random_forest_snia.classifier_sigmoid import get_sigmoid_features_dev
 from fink_science.random_forest_snia.classifier_sigmoid import RF_FEATURE_NAMES
-from fink_science.random_forest_snia.classifier_sigmoid import return_list_of_sn_host
 
 from fink_science.tester import spark_unit_tests
 
+def apply_selection_cuts_ztf(
+        magpsf: pd.Series, ndethist: pd.Series, cdsxmatch: pd.Series,
+        minpoints: int = 4, maxndethist: int = 20) -> pd.Series:
+    """ Apply selection cuts to keep only alerts of interest
+    for early SN Ia analysis
+
+    Parameters
+    ----------
+    magpsf: pd.Series
+        Series containing data measurement (array of double). Each row contains
+        all measurement values for one alert.
+    ndethist: pd.Series
+        Series containing length of the alert history (int).
+        Each row contains the (single) length of the alert.
+    cdsxmatch: pd.Series
+        Series containing crossmatch label with SIMBAD (str).
+        Each row contains one label.
+
+    Returns
+    ---------
+    mask: pd.Series
+        Series containing `True` if the alert is valid, `False` otherwise.
+        Each row contains one boolean.
+    """
+    # Flag alerts with less than 3 points in total
+    mask = magpsf.apply(lambda x: np.sum(np.array(x) == np.array(x))) >= minpoints
+
+    # only alerts with less or equal than 20 measurements
+    mask *= (ndethist.astype(int) <= maxndethist)
+
+    # reject galactic objects
+    list_of_sn_host = return_list_of_eg_host()
+    mask *= cdsxmatch.apply(lambda x: x in list_of_sn_host)
+
+    return mask
+
 @pandas_udf(DoubleType(), PandasUDFType.SCALAR)
 def rfscore_sigmoid_full(jd, fid, magpsf, sigmapsf, cdsxmatch, ndethist, model=None) -> pd.Series:
     """ Return the probability of an alert to be a SNe Ia using a Random
@@ -92,12 +129,12 @@ def rfscore_sigmoid_full(jd, fid, magpsf, sigmapsf, cdsxmatch, ndethist, model=N
     +----------------+-----+
     |rf_snia_vs_nonia|  pIa|
     +----------------+-----+
-    |           0.839|0.839|
-    |           0.782|0.782|
-    |           0.887|0.887|
-    |           0.785|0.785|
-    |            0.88| 0.88|
-    |           0.777|0.777|
+    |           0.839|0.597|
+    |           0.782| 0.62|
+    |           0.887|0.629|
+    |           0.785|0.596|
+    |            0.88|0.641|
+    |           0.777|0.611|
     +----------------+-----+
     <BLANKLINE>
 
@@ -113,40 +150,13 @@ def rfscore_sigmoid_full(jd, fid, magpsf, sigmapsf, cdsxmatch, ndethist, model=N
     >>> df.agg({"pIa": "max"}).collect()[0][0] < 1.0
     True
     """
-    # Flag empty alerts
-    mask = magpsf.apply(lambda x: np.sum(np.array(x) == np.array(x))) > 3
-
-    mask *= (ndethist.astype(int) <= 20)
-
-    list_of_sn_host = return_list_of_sn_host()
-    mask *= cdsxmatch.apply(lambda x: x in list_of_sn_host)
+    mask = apply_selection_cuts_ztf(magpsf, ndethist, cdsxmatch)
 
     if len(jd[mask]) == 0:
         return pd.Series(np.zeros(len(jd), dtype=float))
 
-    # add an exploded column with SNID
-    df_tmp = pd.DataFrame.from_dict(
-        {
-            'jd': jd[mask],
-            'SNID': range(len(jd[mask]))
-        }
-    )
-    df_tmp = df_tmp.explode('jd')
-
-    # compute flux and flux error
-    data = [mag2fluxcal_snana(*args) for args in zip(
-        magpsf[mask].explode(),
-        sigmapsf[mask].explode())]
-    flux, error = np.transpose(data)
-
-    # make a Pandas DataFrame with exploded series
-    pdf = pd.DataFrame.from_dict({
-        'SNID': df_tmp['SNID'],
-        'MJD': df_tmp['jd'],
-        'FLUXCAL': flux,
-        'FLUXCALERR': error,
-        'FLT': fid[mask].explode().replace({1: 'g', 2: 'r'})
-    })
+    candid = pd.Series(range(len(jd)))
+    pdf = format_data_as_snana(jd, magpsf, sigmapsf, fid, candid, mask)
 
     # Load pre-trained model `clf`
     if model is not None:
@@ -157,22 +167,30 @@ def rfscore_sigmoid_full(jd, fid, magpsf, sigmapsf, cdsxmatch, ndethist, model=N
         clf = load_scikit_model(model)
 
     test_features = []
+    flag = []
     for id in np.unique(pdf['SNID']):
         pdf_sub = pdf[pdf['SNID'] == id]
         features = get_sigmoid_features_dev(pdf_sub)
+        if (features[0] == 0) or (features[6] == 0):
+            flag.append(False)
+        else:
+            flag.append(True)
         test_features.append(features)
 
+    flag = np.array(flag, dtype=np.bool)
+
     # Make predictions
     probabilities = clf.predict_proba(test_features)
+    probabilities[~flag] = 0.0
 
     # Take only probabilities to be Ia
     to_return = np.zeros(len(jd), dtype=float)
-    to_return[mask] = probabilities.T[0]
+    to_return[mask] = probabilities.T[1]
 
     return pd.Series(to_return)
 
 @pandas_udf(StringType(), PandasUDFType.SCALAR)
-def extract_features_rf_snia(jd, fid, magpsf, sigmapsf) -> pd.Series:
+def extract_features_rf_snia(jd, fid, magpsf, sigmapsf, cdsxmatch, ndethist) -> pd.Series:
     """ Return the features used by the RF classifier.
 
     There are 12 features. Order is:
@@ -187,6 +205,10 @@ def extract_features_rf_snia(jd, fid, magpsf, sigmapsf) -> pd.Series:
         Filter IDs (int)
     magpsf, sigmapsf: Spark DataFrame Columns
         Magnitude from PSF-fit photometry, and 1-sigma error
+    cdsxmatch: Spark DataFrame Column
+        Type of object found in Simbad (string)
+    ndethist: Spark DataFrame Column
+        Column containing the number of detection by ZTF at 3 sigma (int)
 
     Returns
     ----------
@@ -215,6 +237,7 @@ def extract_features_rf_snia(jd, fid, magpsf, sigmapsf) -> pd.Series:
 
     # Perform the fit + classification (default model)
     >>> args = [F.col(i) for i in what_prefix]
+    >>> args += [F.col('cdsxmatch'), F.col('candidate.ndethist')]
     >>> df = df.withColumn('features', extract_features_rf_snia(*args))
 
     >>> for name in RF_FEATURE_NAMES:
@@ -223,36 +246,15 @@ def extract_features_rf_snia(jd, fid, magpsf, sigmapsf) -> pd.Series:
 
     # Trigger something
     >>> df.agg({RF_FEATURE_NAMES[0]: "min"}).collect()[0][0]
-    -2663.2421875
+    0.0
     """
-    # Flag empty alerts
-    mask = magpsf.apply(lambda x: np.sum(np.array(x) == np.array(x))) > 3
+    mask = apply_selection_cuts_ztf(magpsf, ndethist, cdsxmatch)
+
     if len(jd[mask]) == 0:
         return pd.Series(np.zeros(len(jd), dtype=float))
 
-    # add an exploded column with SNID
-    df_tmp = pd.DataFrame.from_dict(
-        {
-            'jd': jd[mask],
-            'SNID': range(len(jd[mask]))
-        }
-    )
-    df_tmp = df_tmp.explode('jd')
-
-    # compute flux and flux error
-    data = [mag2fluxcal_snana(*args) for args in zip(
-        magpsf[mask].explode(),
-        sigmapsf[mask].explode())]
-    flux, error = np.transpose(data)
-
-    # make a Pandas DataFrame with exploded series
-    pdf = pd.DataFrame.from_dict({
-        'SNID': df_tmp['SNID'],
-        'MJD': df_tmp['jd'],
-        'FLUXCAL': flux,
-        'FLUXCALERR': error,
-        'FLT': fid[mask].explode().replace({1: 'g', 2: 'r'})
-    })
+    candid = pd.Series(range(len(jd)))
+    pdf = format_data_as_snana(jd, magpsf, sigmapsf, fid, candid, mask)
 
     test_features = []
     for id in np.unique(pdf['SNID']):
@@ -269,6 +271,115 @@ def extract_features_rf_snia(jd, fid, magpsf, sigmapsf) -> pd.Series:
 
     return pd.Series(concatenated_features)
 
+@pandas_udf(DoubleType(), PandasUDFType.SCALAR)
+def rfscore_sigmoid_elasticc(midPointTai, filterName, psFlux, psFluxErr, cdsxmatch, nobs, model=None) -> pd.Series:
+    """ Return the probability of an alert to be a SNe Ia using a Random
+    Forest Classifier (sigmoid fit) on ELaSTICC alert data.
+
+    You need to run the SIMBAD crossmatch before.
+
+    Parameters
+    ----------
+    midPointTai: Spark DataFrame Column
+        JD times (vectors of floats)
+    filterName: Spark DataFrame Column
+        Filter IDs (vectors of str)
+    psFlux, psFluxErr: Spark DataFrame Columns
+        SNANA calibrated flux, and 1-sigma error (vectors of floats)
+    cdsxmatch: Spark DataFrame Column
+        Type of object found in Simbad (string)
+    nobs: Spark DataFrame Column
+        Column containing the number of detections by LSST
+    model: Spark DataFrame Column, optional
+        Path to the trained model. Default is None, in which case the default
+        model `data/models/default-model.obj` is loaded.
+
+    Returns
+    ----------
+    probabilities: 1D np.array of float
+        Probability between 0 (non-Ia) and 1 (Ia).
+
+    Examples
+    ----------
+    >>> from fink_science.utilities import concat_col
+    >>> from pyspark.sql import functions as F
+
+    >>> df = spark.read.format('parquet').load(elasticc_alert_sample)
+
+    # Assuming random positions
+    >>> df = df.withColumn('cdsxmatch', F.lit('Unknown'))
+
+    # Required alert columns
+    >>> what = ['midPointTai', 'filterName', 'psFlux', 'psFluxErr']
+
+    # Use for creating temp name
+    >>> prefix = 'c'
+    >>> what_prefix = [prefix + i for i in what]
+
+    # Append temp columns with historical + current measurements
+    >>> for colname in what:
+    ...     df = concat_col(
+    ...         df, colname, prefix=prefix,
+    ...         current='diaSource', history='prvDiaSources')
+
+    # Perform the fit + classification (default model)
+    >>> args = [F.col(i) for i in what_prefix]
+    >>> args += [F.col('cdsxmatch'), F.col('diaSource.nobs')]
+    >>> df = df.withColumn('pIa', rfscore_sigmoid_elasticc(*args))
+
+    >>> df.filter(df['pIa'] > 0.0).count()
+    29
+    """
+    mask = apply_selection_cuts_ztf(psFlux, nobs, cdsxmatch, maxndethist=100)
+
+    mask *= filterName.apply(lambda array: np.sum([x in ['g', 'r'] for x in array]) > 3)
+
+    if len(midPointTai[mask]) == 0:
+        return pd.Series(np.zeros(len(midPointTai), dtype=float))
+
+    # change filter name for the moment to stick to ZTF definition
+    filter_conversion_dic = {'u': 0, 'g': 1, 'r': 2, 'i': 3, 'z': 4, 'Y': 5}
+    filterName = filterName.apply(lambda array: [filter_conversion_dic[x] for x in array])
+
+    candid = pd.Series(range(len(midPointTai)))
+    pdf = format_data_as_snana(
+        midPointTai, psFlux, psFluxErr,
+        filterName, candid, mask,
+        transform_to_flux=False
+    )
+
+    # Load pre-trained model `clf`
+    if model is not None:
+        clf = load_scikit_model(model.values[0])
+    else:
+        curdir = os.path.dirname(os.path.abspath(__file__))
+        model = curdir + '/data/models/default-model_sigmoid.obj'
+        clf = load_scikit_model(model)
+
+    test_features = []
+    flag = []
+    for id in np.unique(pdf['SNID']):
+        f1 = pdf['SNID'] == id
+        pdf_sub = pdf[f1]
+        features = get_sigmoid_features_dev(pdf_sub)
+        if (features[0] == 0) or (features[6] == 0):
+            flag.append(False)
+        else:
+            flag.append(True)
+        test_features.append(features)
+
+    flag = np.array(flag, dtype=np.bool)
+
+    # Make predictions
+    probabilities = clf.predict_proba(test_features)
+    probabilities[~flag] = 0.0
+
+    # Take only probabilities to be Ia
+    to_return = np.zeros(len(midPointTai), dtype=float)
+    to_return[mask] = probabilities.T[1]
+
+    return pd.Series(to_return)
+
 
 if __name__ == "__main__":
     """ Execute the test suite """
@@ -279,6 +390,9 @@ if __name__ == "__main__":
     ztf_alert_sample = 'file://{}/data/alerts/datatest'.format(path)
     globs["ztf_alert_sample"] = ztf_alert_sample
 
+    elasticc_alert_sample = 'file://{}/data/alerts/elasticc_parquet'.format(path)
+    globs["elasticc_alert_sample"] = elasticc_alert_sample
+
     model_path_sigmoid = '{}/data/models/default-model_sigmoid.obj'.format(path)
     globs["model_path_sigmoid"] = model_path_sigmoid
 
